benchmark

package
v0.0.0-...-d3dc1b7 Latest Latest
Warning

This package is not in the latest version of its module.

Go to latest
Published: Apr 17, 2026 License: AGPL-3.0 Imports: 13 Imported by: 0

Documentation

Index

Constants

This section is empty.

Variables

This section is empty.

Functions

func DeleteContainers

func DeleteContainers(c lxd.InstanceServer, containers []api.Instance, parallel int) (time.Duration, error)

DeleteContainers removes containers created by the benchmark.

func GetContainers

func GetContainers(c lxd.InstanceServer) ([]api.Instance, error)

GetContainers returns containers created by the benchmark.

func LaunchContainers

func LaunchContainers(c lxd.InstanceServer, count int, parallel int, image string, privileged bool, start bool, freeze bool) (time.Duration, error)

LaunchContainers launches a set of containers.

func PrintServerInfo

func PrintServerInfo(c lxd.InstanceServer) error

PrintServerInfo prints out information about the server.

func StartContainers

func StartContainers(c lxd.InstanceServer, containers []api.Instance, parallel int) (time.Duration, error)

StartContainers starts containers created by the benchmark.

func StopContainers

func StopContainers(c lxd.InstanceServer, containers []api.Instance, parallel int) (time.Duration, error)

StopContainers stops containers created by the benchmark.

Types

type CSVReport

type CSVReport struct {
	Filename string
	// contains filtered or unexported fields
}

CSVReport reads/writes a CSV report file.

func (*CSVReport) AddRecord

func (r *CSVReport) AddRecord(label string, elapsed time.Duration) error

AddRecord adds a record to the report.

func (*CSVReport) Load

func (r *CSVReport) Load() (err error)

Load reads current content of the filename and loads records.

func (*CSVReport) Write

func (r *CSVReport) Write() (err error)

Write writes current records to file.

Jump to