openfga

package
v0.0.0-...-a7f6617 Latest Latest
Warning

This package is not in the latest version of its module.

Go to latest
Published: Dec 16, 2025 License: AGPL-3.0 Imports: 18 Imported by: 0

Documentation

Index

Constants

This section is empty.

Variables

This section is empty.

Functions

func NewOpenFGAStore

func NewOpenFGAStore(clusterDB *db.Cluster) storage.OpenFGADatastore

NewOpenFGAStore returns a new storage.OpenFGADatastore that is backed directly by the dqlite database.

Types

type RequestCache

type RequestCache struct {
	// contains filtered or unexported fields
}

RequestCache should be set in the request context to allow the OpenFGADatastore implementation to reduce the number of database calls that are made on a per-request basis.

Jump to