linux

package
v0.0.0-...-8bf450a Latest Latest
Warning

This package is not in the latest version of its module.

Go to latest
Published: May 14, 2026 License: AGPL-3.0 Imports: 6 Imported by: 0

Documentation

Index

Constants

const ABSTRACT_UNIX_SOCK_LEN int = C.ABSTRACT_UNIX_SOCK_LEN //nolint:revive
const CLOSE_RANGE_CLOEXEC uint32 = C.CLOSE_RANGE_CLOEXEC //nolint:revive
const CLOSE_RANGE_UNSHARE uint32 = C.CLOSE_RANGE_UNSHARE //nolint:revive
const IoctlBlkZname = C.BLKZNAME

IoctlBlkZname matches BLKZNAME (ZFS specific).

const IoctlBtrfsSetReceivedSubvol = C.BTRFS_IOC_SET_RECEIVED_SUBVOL

IoctlBtrfsSetReceivedSubvol is used to set information about a received subvolume.

const IoctlHIDIOCGrawInfo = C.HIDIOCGRAWINFO

IoctlHIDIOCGrawInfo contains the bus type, the vendor ID (VID), and product ID (PID) of the device.

const IoctlVhostVsockSetGuestCid = C.VHOST_VSOCK_SET_GUEST_CID

IoctlVhostVsockSetGuestCid is used to set the vsock guest context ID.

const ZFSMaxDatasetNameLen = C.ZFS_MAX_DATASET_NAME_LEN

ZFSMaxDatasetNameLen is the maximum length of a ZFS dataset name.

Variables

This section is empty.

Functions

func CloseRange

func CloseRange(FirstFd uint32, LastFd uint32, Flags uint32) error

CloseRange closes a range of file descriptors.

func CreateMemfd

func CreateMemfd(content []byte) (*os.File, error)

CreateMemfd creates a new memfd for the provided byte slice.

func PidFdOpen

func PidFdOpen(Pid int, Flags uint32) (*os.File, error)

PidFdOpen opens a PID file descriptor for a given PID.

func PidfdSendSignal

func PidfdSendSignal(Pidfd int, Signal int, Flags uint32) error

PidfdSendSignal sends a signal to a process using its PID file descriptor.

func ReadPid

func ReadPid(r *os.File) int

ReadPid reads the PID from a file descriptor.

Types

This section is empty.

Jump to