Documentation
¶
Index ¶
Constants ¶
This section is empty.
Variables ¶
This section is empty.
Functions ¶
This section is empty.
Types ¶
type API10Put ¶
type API10Put struct {
// Context ID
// Example: 2
CID uint32 `json:"cid" yaml:"cid"`
// Port of the vsock server
// Example: 1234
Port uint32 `json:"port" yaml:"port"`
// Server certificate as PEM encoded X509
// Example: X509 PEM certificate
Certificate string `json:"certificate" yaml:"certificate"`
// Whether or not to enable devlxd
// Example: true
Devlxd bool `json:"devlxd" yaml:"devlxd"`
}
API10Put contains the fields which are needed for the lxd-agent to connect to LXD.
type DeviceEventAction ¶
type DeviceEventAction string
DeviceEventAction represents a device event action for instance devices.
const ( DeviceAdded DeviceEventAction = "added" DeviceRemoved DeviceEventAction = "removed" DeviceUpdated DeviceEventAction = "updated" )
All supported device events for instance devices.
Click to show internal directories.
Click to hide internal directories.