Documentation
¶
Index ¶
Constants ¶
const ( // DriverEmbeddedOpenFGA is the default authorization driver. It currently falls back to DriverTLS for all TLS // clients. It cannot be initialised until after the cluster database is operational. DriverEmbeddedOpenFGA string = "embedded-openfga" )
const ( // DriverTLS is used at start up to allow communication between cluster members and initialise the cluster database. DriverTLS string = "tls" )
Variables ¶
var ErrUnknownDriver = errors.New("Unknown driver")
ErrUnknownDriver is the "Unknown driver" error.
Functions ¶
func LoadAuthorizer ¶
func LoadAuthorizer(ctx context.Context, driver string, logger logger.Logger, options ...func(opts *Opts)) (auth.Authorizer, error)
LoadAuthorizer instantiates, configures, and initialises an Authorizer.
func WithOpenFGADatastore ¶
func WithOpenFGADatastore(store storage.OpenFGADatastore) func(*Opts)
WithOpenFGADatastore should be passed into LoadAuthorizer when using the embedded openfga driver.
Types ¶
Click to show internal directories.
Click to hide internal directories.