Documentation
¶
Index ¶
Constants ¶
This section is empty.
Variables ¶
This section is empty.
Functions ¶
func NewOpenFGAStore ¶
func NewOpenFGAStore(clusterDB *db.Cluster) storage.OpenFGADatastore
NewOpenFGAStore returns a new storage.OpenFGADatastore that is backed directly by the dqlite database.
Types ¶
type RequestCache ¶
type RequestCache struct {
// contains filtered or unexported fields
}
RequestCache should be set in the request context to allow the OpenFGADatastore implementation to reduce the number of database calls that are made on a per-request basis.
Click to show internal directories.
Click to hide internal directories.