Documentation
¶
Index ¶
Constants ¶
This section is empty.
Variables ¶
This section is empty.
Functions ¶
Types ¶
type ConflictAction ¶
type ConflictAction int
ConflictAction returns the action to take if a conflicting operation is already running.
const ( // ConflictActionNone means operation has no conflicts, all operations of this type can run concurrently. ConflictActionNone ConflictAction = iota // ConflictActionFail asks to resolve conflicts by failing to create a new operation if a conflicting operation is already running. ConflictActionFail )
type Type ¶
type Type int64
Type is a numeric code identifying the type of Operation.
const ( Unknown Type = iota ClusterBootstrap ClusterJoin BackupCreate BackupRename BackupRestore BackupRemove ConsoleShow InstanceCreate InstanceUpdate InstanceRename InstanceMigrate InstanceLiveMigrate InstanceFreeze InstanceUnfreeze InstanceDelete InstanceStart InstanceStop InstanceRestart InstanceRebuild CommandExec SnapshotCreate SnapshotRename SnapshotRestore SnapshotTransfer SnapshotUpdate SnapshotDelete SnapshotCopy ImageDownload ImageDelete ImageDownloadToken ImageRefresh VolumeCopy VolumeCreate VolumeMigrate VolumeMove VolumeSnapshotCopy VolumeSnapshotCreate VolumeSnapshotDelete VolumeSnapshotUpdate ProjectRename ImagesExpire ImagesPruneLeftover ImagesUpdate ImagesSynchronize LogsExpire InstanceTypesUpdate BackupsExpire SnapshotsExpire CustomVolumeSnapshotsExpire CustomVolumeBackupCreate CustomVolumeBackupRemove CustomVolumeBackupRename CustomVolumeBackupRestore WarningsPruneResolved ClusterJoinToken VolumeSnapshotRename ClusterMemberEvacuate ClusterMemberRestore CertificateAddToken RemoveOrphanedOperations RenewServerCertificate RemoveExpiredTokens ClusterHeal RemoveExpiredOIDCSessions ProfileUpdate VolumeUpdate VolumeDelete ImageUploadToken ImageUpload InstanceCopy VolumeSnapshotsCreateScheduled InstanceStateUpdateBulk VolumeSnapshotTransfer ProjectDelete Wait SnapshotsCreateScheduled )
Possible values for Type
WARNING: The type codes are stored in the database, so this list of definitions should be normally append-only. Any other change requires a database update.
func (Type) ConflictAction ¶
func (t Type) ConflictAction() ConflictAction
ConflictAction returns the action to take if a conflicting operation is already running.
func (Type) Description ¶
Description return a human-readable description of the operation type.
func (Type) EntityType ¶
EntityType returns the primary entity.Type that the Type operates on.
Click to show internal directories.
Click to hide internal directories.