Documentation
¶
Index ¶
Constants ¶
This section is empty.
Variables ¶
This section is empty.
Functions ¶
Types ¶
type NetworkZone ¶
type NetworkZone interface {
// Info.
ID() int64
Project() string
Info() *api.NetworkZone
Etag() []any
UsedBy(ctx context.Context) ([]string, error)
Content(ctx context.Context) (*strings.Builder, error)
SOA() (*strings.Builder, error)
// Records.
AddRecord(ctx context.Context, req api.NetworkZoneRecordsPost) error
GetRecords(ctx context.Context) ([]api.NetworkZoneRecord, error)
GetRecord(ctx context.Context, name string) (*api.NetworkZoneRecord, error)
UpdateRecord(ctx context.Context, name string, req api.NetworkZoneRecordPut) error
DeleteRecord(ctx context.Context, name string) error
// Modifications.
Update(config *api.NetworkZonePut, clientType request.ClientType) error
Delete(ctx context.Context) error
// contains filtered or unexported methods
}
NetworkZone represents a Network zone.
func LoadByName ¶
LoadByName loads and initialises a Network zone from the database by name.
func LoadByNameAndProject ¶
func LoadByNameAndProject(ctx context.Context, s *state.State, projectName string, name string) (NetworkZone, error)
LoadByNameAndProject loads and initialises a Network zone from the database by project and name.
Click to show internal directories.
Click to hide internal directories.