Documentation
¶
Index ¶
Constants ¶
This section is empty.
Variables ¶
This section is empty.
Functions ¶
Types ¶
type HTTPRequestCanceller ¶
type HTTPRequestCanceller struct {
// contains filtered or unexported fields
}
HTTPRequestCanceller tracks a cancelable operation.
func NewHTTPRequestCanceller ¶
func NewHTTPRequestCanceller() *HTTPRequestCanceller
NewHTTPRequestCanceller returns a new HTTPRequestCanceller struct.
func NewHTTPRequestCancellerWithContext ¶
func NewHTTPRequestCancellerWithContext(ctx context.Context) *HTTPRequestCanceller
NewHTTPRequestCancellerWithContext returns a new HTTPRequestCanceller that automatically cancels when the given context is cancelled.
func (*HTTPRequestCanceller) Cancel ¶
func (c *HTTPRequestCanceller) Cancel() error
Cancel will attempt to cancel all ongoing operations.
func (*HTTPRequestCanceller) Cancelable ¶
func (c *HTTPRequestCanceller) Cancelable() bool
Cancelable indicates whether there are operations that support cancellation.
Click to show internal directories.
Click to hide internal directories.