Documentation
¶
Index ¶
- Variables
- func Mirror(conn *websocket.Conn, rwc io.ReadWriteCloser) (chRead chan error, chWrite chan error)
- func MirrorRead(conn *websocket.Conn, rc io.Reader) chan error
- func MirrorWrite(conn *websocket.Conn, wc io.Writer) chan error
- func NewWrapper(conn *websocket.Conn) io.ReadWriteCloser
- func Proxy(source *websocket.Conn, target *websocket.Conn) chan struct{}
- func StartKeepAlive(conn *websocket.Conn)
Constants ¶
This section is empty.
Variables ¶
var Upgrader = websocket.Upgrader{ CheckOrigin: checkOrigin, HandshakeTimeout: time.Second * 5, }
Upgrader is a websocket upgrader that validates the Origin header.
Functions ¶
func Mirror ¶
Mirror takes a websocket and replicates all read/write to a ReadWriteCloser. Returns channels indicating when reads and writes are finished (respectively).
func MirrorRead ¶
MirrorRead is a uni-directional mirror which replicates an io.Reader to a websocket.
func MirrorWrite ¶
MirrorWrite is a uni-directional mirror which replicates a websocket to an io.Writer.
func NewWrapper ¶
func NewWrapper(conn *websocket.Conn) io.ReadWriteCloser
NewWrapper returns a new ReadWriteCloser wrapper for a websocket connection.
func StartKeepAlive ¶
StartKeepAlive sets TCP_USER_TIMEOUT and TCP keep alive timeouts on a connection and starts a periodic websocket ping go routine if the underlying connection is TCP. Otherwise this is a no-op.
Types ¶
This section is empty.